# -*- coding: utf-8 -*-

from enum import Enum

from wallabag.api.api import Api, ApiMethod, ValueException


class Params(Enum):
    TITLE = "title"
    STARRED = "starred"
    READ = "read"


class AddEntry(Api):

    API_METHOD = ApiMethod.ADD_ENTRY

    class ApiParams(Enum):
        URL = "url"
        TITLE = "title"
        STARRED = "starred"
        ARCHIVE = "archive"

    def __init__(self, config, url, params):
        Api.__init__(self, config)
        self.url = url
        self.params = params

    def _make_request(self, request):
        return self._request_post(request)

    def _get_api_url(self):
        return self._build_url(AddEntry.API_METHOD)

    def _get_data(self):
        ApiParams = self.ApiParams
        data = {
            ApiParams.URL.value: self._validate_url(self.url)
        }
        if Params.TITLE in self.params and self.params[Params.TITLE]:
            data[ApiParams.TITLE.value] = self.params[Params.TITLE]
        if Params.STARRED in self.params:
            self._put_bool_param(data, Params.STARRED, ApiParams.STARRED)
        if Params.READ in self.params:
            self._put_bool_param(data, Params.READ, ApiParams.ARCHIVE)
        if not data:
            raise ValueException("The data object is empty")
        return data



# -*- coding: utf-8 -*-

from colorama import Fore

from wallabag.api.api import Response, RequestException
from wallabag.api.get_entry import GetEntry
from wallabag.commands.show import ShowCommand, ShowCommandParams
from wallabag.config import Configs


class TestShowCommand():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_entry_not_exist(self, monkeypatch):
        def request(self):
            response = Response(404, None)
            raise RequestException(
                    response.error_text, response.error_description)
        monkeypatch.setattr(GetEntry, 'request', request)

        result, output = ShowCommand(
                self.config, ShowCommandParams(1000)).run()
        assert not result
        assert output == "Error: 404: API was not found."

    def test_entry_content(self, monkeypatch):
        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)

        params = ShowCommandParams(1)
        result, output = ShowCommand(self.config, params).run()
        assert result
        assert output == 'title\n\n\ncontent'

    def test_entry_html_content(self, monkeypatch):
        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "<h1>head</h1>content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)

        params = ShowCommandParams(1, html=True)
        result, output = ShowCommand(self.config, params).run()
        assert result
        assert output == 'title\n\n\n<h1>head</h1>content'

    def test_entry_html_strip_content(self, monkeypatch):
        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "<h1>head</h1>content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)

        params = ShowCommandParams(1, html=False, colors=False)
        result, output = ShowCommand(self.config, params).run()
        assert result
        assert output == 'title\n\n\n\n\nhead\ncontent'

    def test_entry_html_strip_content_with_colors(self, monkeypatch):
        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "<h1>head</h1>content",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)

        params = ShowCommandParams(1, html=False, colors=True)
        result, output = ShowCommand(self.config, params).run()
        assert result
        assert output == f'title\n\n\n\n\n{Fore.BLUE}head{Fore.RESET}\ncontent'

    def test_entry_html_image_content(self, monkeypatch):
        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content":\
                            "<h1>head</h1>content<img alt=\\"Message desc\\"\
                            src=\\"https://imag.es/1.jpg\\" />",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)

        params = ShowCommandParams(1, html=False, colors=False)
        result, output = ShowCommand(self.config, params).run()
        assert result
        assert output == 'title\n\n\n\n\nhead\ncontent[IMAGE "Message desc"]'

    def test_entry_html_image_content_with_links(self, monkeypatch):
        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content":\
                            "<h1>head</h1>content<img alt=\\"Message desc\\"\
                            src=\\"https://imag.es/1.jpg\\" />",\
                            "url": "url", "is_archived": 0, "is_starred": 1}')

        monkeypatch.setattr(GetEntry, 'request', request)

        params = ShowCommandParams(
                1, html=False, colors=False, image_links=True)
        result, output = ShowCommand(self.config, params).run()
        assert result
        assert output == 'title\n\n\n\n\nhead\ncontent\
[IMAGE "Message desc" (https://imag.es/1.jpg)]'



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import Response, RequestException
from wallabag.api.get_entry import GetEntry
from wallabag.api.update_entry import UpdateEntry
from wallabag.commands.update import UpdateCommand, UpdateCommandParams
from wallabag.config import Configs


class TestUpdateCommand():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_not_existed_entry(self, monkeypatch):
        def request(self):
            response = Response(404, None)
            raise RequestException(
                    response.error_text, response.error_description)

        monkeypatch.setattr(GetEntry, 'request', request)

        params = UpdateCommandParams(1)
        params.toggle_read = True
        result, output = UpdateCommand(self.config, params).run()
        assert not result
        assert output == "Error: 404: API was not found."

    def test_no_parameters_given(self):
        result, output = UpdateCommand(
                self.config, UpdateCommandParams(1)).run()
        assert not result
        assert output == 'Error: No parameter given.'

    @pytest.mark.parametrize('values', [
        ((0, 0), (1, 0), (True, False)),
        ((1, 0), (0, 0), (True, False)),
        ((0, 1), (0, 0), (False, True)),
        ((0, 0), (0, 1), (False, True)),
        ])
    def test_toggle_parameters(self, monkeypatch, values):
        make_request_runned = False

        def _make_request(self, request):
            nonlocal make_request_runned
            make_request_runned = True
            if values[1][0] == 1:
                assert request.data[UpdateEntry.ApiParams.ARCHIVE.value]\
                        == values[1][0]
            if values[1][1] == 1:
                assert request.data[UpdateEntry.ApiParams.STARRED.value]\
                        == values[1][1]
            return Response(200)

        def request(self):
            return Response(
                    200, '{"id": 1, "title": "title", "content": "content",\
                            "url": "url", "is_archived": %s, "is_starred": %s}'
                    % (values[0][0], values[0][1]))

        monkeypatch.setattr(GetEntry, 'request', request)
        monkeypatch.setattr(UpdateEntry, '_make_request', _make_request)

        params = UpdateCommandParams(1)
        params.toggle_read = values[2][0]
        params.toggle_star = values[2][1]
        result, output = UpdateCommand(self.config, params).run()
        assert result
        assert make_request_runned



# -*- coding: utf-8 -*-

from wallabag.api.api import Response
from wallabag.api.add_entry import AddEntry
from wallabag.api.entry_exists import EntryExists
from wallabag.commands.add import AddCommand, AddCommandParams
from wallabag.config import Configs


class TestAddCommand():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_entry_existed_url(self, monkeypatch):
        make_request_runned = False
        url = "http://test/url"

        def _make_request(self, request):
            nonlocal make_request_runned
            make_request_runned = True
            assert request.api_params[EntryExists.ApiParams.URL.value] == url
            return Response(200, '{"exists": 1}')

        monkeypatch.setattr(EntryExists, '_make_request', _make_request)

        result = AddCommand(self.config, AddCommandParams(url)).run()
        assert make_request_runned
        assert result[0]
        assert result[1] == "The url was already saved."

    def test_add_existed_url(self, monkeypatch):

        def entry_exists(self):
            return Response(200, '{"exists": 1}')

        monkeypatch.setattr(EntryExists, 'request', entry_exists)

        params = AddCommandParams('url')
        result = AddCommand(self.config, params).run()
        assert result[0]
        assert result[1] == "The url was already saved."

    def test_add_command_params(self, monkeypatch):
        make_request_runned = False
        url = "http://test/url"
        title = 'test title'
        starred = True
        read = False

        def entry_not_exists(self):
            return Response(200, '{"exists": 0}')

        def _validate_url(self, url):
            return url

        def _make_request(self, request):
            nonlocal make_request_runned
            make_request_runned = True
            assert request.data[AddEntry.ApiParams.URL.value] == url
            assert request.data[AddEntry.ApiParams.ARCHIVE.value] == 0
            assert request.data[AddEntry.ApiParams.STARRED.value] == 1
            assert request.data[AddEntry.ApiParams.TITLE.value] == title
            return Response(200, None)

        monkeypatch.setattr(EntryExists, 'request', entry_not_exists)
        monkeypatch.setattr(AddEntry, '_make_request', _make_request)
        monkeypatch.setattr(AddEntry, '_validate_url', _validate_url)

        params = AddCommandParams(url, title, read=read, starred=starred)
        result = AddCommand(self.config, params).run()
        assert make_request_runned
        assert result[0]
        assert result[1] == "Entry successfully added."



# -*- coding: utf-8 -*-

from wallabag.api.api import ApiException
from wallabag.api.add_entry import AddEntry, Params
from wallabag.api.entry_exists import EntryExists
from wallabag.commands.command import Command


class AddCommandParams():
    target_url = None
    title = None
    starred = None
    read = None

    def __init__(self, target_url, title=None, starred=None, read=None):
        self.target_url = target_url
        self.title = title
        self.starred = starred
        self.read = read


class AddCommand(Command):

    def __init__(self, config, params=None):
        self.config = config
        self.params = params

    def run(self):
        params = self.params
        try:
            api = EntryExists(self.config, params.target_url)
            if api.request().response['exists']:
                return True, "The url was already saved."

            AddEntry(self.config, params.target_url, {
                Params.TITLE: params.title,
                Params.READ: params.read,
                Params.STARRED: params.starred
            }).request()
            return True, "Entry successfully added."
        except ApiException as ex:
            return False, str(ex)



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import ApiMethod, ValueException
from wallabag.api.add_entry import AddEntry, Params
from wallabag.config import Configs


def _is_valid_url(self, url):
    return True


class TestAddEntry():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    @pytest.mark.parametrize('entry_id', [10, "10"])
    def test_api_url(self, entry_id):
        api = AddEntry(self.config, "url", {})

        url = "url" + ApiMethod.ADD_ENTRY.value
        assert url == api._get_api_url()

    @pytest.mark.parametrize('values', [
        (Params.TITLE, "custom title",
            AddEntry.ApiParams.TITLE, "custom title"),
        (Params.READ, True, AddEntry.ApiParams.ARCHIVE, 1),
        (Params.READ, False, AddEntry.ApiParams.ARCHIVE, 0),
        (Params.STARRED, True, AddEntry.ApiParams.STARRED, 1),
        (Params.STARRED, False, AddEntry.ApiParams.STARRED, 0),
        ])
    def test_params(self, monkeypatch, values):
        monkeypatch.setattr(AddEntry, '_is_valid_url', _is_valid_url)

        api = AddEntry(self.config, "url", {
            values[0]: values[1]
        })

        data = api._get_data()
        assert data[values[2].value] == values[3]

    def test_empty_params(self, monkeypatch):
        monkeypatch.setattr(AddEntry, '_is_valid_url', _is_valid_url)

        api = AddEntry(self.config, "", {
            Params.TITLE: "",
            Params.STARRED: None
        })

        with pytest.raises(ValueException):
            api._get_data()

    def test_mixed_params(self, monkeypatch):
        monkeypatch.setattr(AddEntry, '_is_valid_url', _is_valid_url)

        api = AddEntry(self.config, "url", {
            Params.TITLE: "",
            Params.READ: True
        })

        data = api._get_data()
        assert AddEntry.ApiParams.TITLE.value not in data
        assert data[AddEntry.ApiParams.ARCHIVE.value] == 1
        assert 'https://url' == data[AddEntry.ApiParams.URL.value]



# -*- coding: utf-8 -*-

import base64
import configparser
import getpass
import os
import socket
import time
from pathlib import PurePath

from Crypto.Cipher import AES
from Crypto.Hash import MD5

from xdg.BaseDirectory import xdg_config_home as XDG_CONFIG_HOME


CONFIG_DIRECTORY = os.path.expanduser(XDG_CONFIG_HOME)
CONFIG_WALLABAG_DIR = "wallabag-cli"
CONFIG_FILENAME = "config.ini"


class Sections():
    API = "api"
    OAUTH2 = "oauth2"
    TOKEN = "token"


class Options():
    SERVERURL = "serverurl"
    USERNAME = "username"
    PASSWORD = "password"

    CLIENT = "client"
    SECRET = "secret"

    ACCESS_TOKEN = "access_token"
    EXPIRES = "expires"


class Configs():

    LOAD_ERROR =\
        """
        Error: Could not load or create the config file.
        """
    ENCRYPT_VALUES = [
            (Sections.API, Options.PASSWORD),
            (Sections.OAUTH2, Options.SECRET)]

    custom_path = None

    def __init__(self, path=None):
        self.config = configparser.ConfigParser()
        self.load(path)

    def get_path(self, custom_path=None):
        if custom_path:
            self.custom_path = custom_path
        if self.custom_path:
            return self.custom_path
        return PurePath(CONFIG_DIRECTORY,
                        CONFIG_WALLABAG_DIR,
                        CONFIG_FILENAME)

    def set(self, section, name, value):
        if not self.config.has_section(section):
            self.config.add_section(section)
        if (section, name) in Configs.ENCRYPT_VALUES:
            value = self.__encrypt(value)
        self.config.set(section, name, value)

    def get(self, section, name, fallback=None):
        value = self.config.get(section, name, fallback=fallback)
        if (section, name) in Configs.ENCRYPT_VALUES:
            value = self.__decrypt(value)
        return value

    def getint(self, section, name, fallback=0):
        return self.config.getint(section, name, fallback=fallback)

    def getfloat(self, section, name, fallback=0):
        return self.config.getfloat(section, name, fallback=fallback)

    def is_token_expired(self):
        return self.getfloat('token', 'expires', 0) - time.time() < 0

    def is_valid(self):
        options_to_check = ((Sections.API, Options.SERVERURL),
                            (Sections.API, Options.USERNAME),
                            (Sections.API, Options.PASSWORD),
                            (Sections.OAUTH2, Options.CLIENT),
                            (Sections.OAUTH2, Options.SECRET))
        for s, o in options_to_check:
            if not self.config.has_option(s, o):
                return False
        return True

    def save(self, custom_path=None):
        path = self.get_path(custom_path)

        if not os.path.exists(path.parents[0]):
            os.makedirs(path.parents[0])
        with open(path, mode='w') as file:
            self.config.write(file)

    def load(self, custom_path=None):
        path = self.get_path(custom_path)
        try:
            self.config.read(path)
        except configparser.Error as e:
            print(e)
            raise ValueError(Configs.LOAD_ERROR)

    def load_or_create(self, custom_path=None):
        path = self.get_path(custom_path)

        success = False
        if not os.path.exists(path):
            success = self.save(path)
        else:
            success = self.load(path)
        if not success:
            raise ValueError(Configs.LOAD_ERROR)

    def __cryptkey(self):
        s1 = getpass.getuser()
        s2 = socket.gethostname()
        return MD5.new((s1 + s2).encode("utf-8")).digest()

    def __encrypt(self, value):
        try:
            cipher = AES.new(self.__cryptkey(), AES.MODE_EAX)
            ciphertext, tag = cipher.encrypt_and_digest(value.encode('utf-8'))
            return "%s@%s@%s" % (
                    base64.b64encode(cipher.nonce).decode('utf-8'),
                    base64.b64encode(ciphertext).decode('utf-8'),
                    base64.b64encode(tag).decode('utf-8'))
        except Exception as e:
            print(e)
            ret = None
        return ret

    def __decrypt(self, value):
        try:
            nonce, ciphertext, tag = map(
                    lambda v: base64.b64decode(v), value.split('@'))
            cipher = AES.new(self.__cryptkey(), AES.MODE_EAX, nonce)
            return cipher.decrypt_and_verify(ciphertext, tag).decode('utf-8')
        except Exception as e:
            print(e)
            ret = None
        return ret



# -*- coding: utf-8 -*-

import click
from colorama import Back

from wallabag.api.api import ApiException
from wallabag.api.get_entry import GetEntry
from wallabag.api.delete_entry import DeleteEntry
from wallabag.commands.command import Command
from wallabag.entry import Entry


class DeleteCommandParams():

    def __init__(self, entry_id, force=False, quiet=False):
        self.entry_id = entry_id
        self.force = force
        self.quiet = quiet


class DeleteCommand(Command):

    WARN_MSG = 'You are going to remove the following entry'

    def __init__(self, config, params):
        self.config = config
        self.params = params or DeleteCommandParams()

    def run(self):
        if not self.params.force:
            try:
                request = GetEntry(self.config, self.params.entry_id).request()
                entr = Entry(request.response)
                confirm_msg = (
                        f"{Back.RED}{DeleteCommand.WARN_MSG}:{Back.RESET}\n\n"
                        f"\t{entr.title}\n\n"
                        "Continue?")
                if not click.confirm(confirm_msg):
                    return True, 'Cancelling'
            except ApiException as ex:
                return False, str(ex)

        try:
            request = DeleteEntry(self.config, self.params.entry_id).request()
            if not self.params.quiet:
                return True, "Entry successfully deleted."
            return True, None
        except ApiException as ex:
            return False, str(ex)



# -*- coding: utf-8 -*-

import pytest

from wallabag.api.api import ApiMethod
from wallabag.api.get_list_entries import GetListEntries, Params
from wallabag.config import Configs


class TestGetEntriesList():

    def setup_method(self, method):
        self.config = Configs("/tmp/config")
        self.config.config.read_string("""
                [api]
                serverurl = url
                username = user
                password = pass
                [oauth2]
                client = 100
                secret = 100
                """)

    def test_api_url(self):
        api = GetListEntries(self.config, {
            Params.COUNT: 10
        })

        assert "url" + ApiMethod.LIST_ENTRIES.value == api._get_api_url()

    @pytest.mark.parametrize('count', [
        (10, 10), (None, 0), (-10, 0)
        ])
    def test_params_count(self, count):
        api = GetListEntries(self.config, {
            Params.COUNT: count[0]
        })

        params = api._get_params()
        assert GetListEntries.ApiParams.PER_PAGE.value in params
        assert params.get(GetListEntries.ApiParams.PER_PAGE.value) == count[1]

    @pytest.mark.parametrize('values', [
        ((
            Params.FILTER_READ,
            GetListEntries.ApiParams.ARCHIVE
            ), True, 1),
        ((
            Params.FILTER_READ,
            GetListEntries.ApiParams.ARCHIVE
            ), None, None),
        ((
            Params.FILTER_READ,
            GetListEntries.ApiParams.ARCHIVE
            ), False, 0),
        ((
            Params.FILTER_READ,
            GetListEntries.ApiParams.ARCHIVE
            ), 'foo', None),
        ((
            Params.FILTER_STARRED,
            GetListEntries.ApiParams.STARRED
            ), True, 1),
        ((
            Params.FILTER_STARRED,
            GetListEntries.ApiParams.STARRED
            ), None, None),
        ((
            Params.FILTER_STARRED,
            GetListEntries.ApiParams.STARRED
            ), False, 0),
        ((
            Params.FILTER_STARRED,
            GetListEntries.ApiParams.STARRED
            ), 'foo', None)
        ])
    def test_bool_params(self, values):
        api = GetListEntries(self.config, {
            Params.COUNT: 1,
            values[0][0]: values[1]
        })

        params = api._get_params()
        if values[2] is not None:
            assert values[0][1].value in params
            assert params.get(values[0][1].value) == values[2]
        else:
            assert values[0][1].value not in params

    @pytest.mark.parametrize('oldest', [
        (True, GetListEntries.ApiValues.ORDER.value.ASC.value),
        (None, None), (False, None)
        ])
    def test_param_oldest(self, oldest):
        api = GetListEntries(self.config, {
            Params.COUNT: 1,
            Params.OLDEST: oldest[0]
        })

        params = api._get_params()
        if oldest[1]:
            assert GetListEntries.ApiParams.ORDER.value in params
            assert oldest[1] == params.get(
                    GetListEntries.ApiParams.ORDER.value)
        else:
            assert GetListEntries.ApiParams.ORDER.value not in params



# -*- coding: utf-8 -*-

from enum import Enum

from wallabag.api.api import Api, ApiMethod, ValueException


class EntryExists(Api):

    API_METHOD = ApiMethod.ENTRY_EXISTS

    class ApiParams(Enum):
        URL = "url"

    def __init__(self, config, url):
        Api.__init__(self, config)
        self.url = url

    def _make_request(self, request):
        return self._request_get(request)

    def _get_api_url(self):
        return self._build_url(EntryExists.API_METHOD)

    def _get_params(self):
        if not self.url:
            raise ValueException("URL is empty")
        return {
            EntryExists.ApiParams.URL.value: self.url
        }



# -*- coding: utf-8 -*-

from enum import Enum

from wallabag.api.api import Api, ApiMethod, ValueException


class Params(Enum):
    TITLE = "title"
    STAR = "star"
    READ = "read"


class UpdateEntry(Api):

    API_METHOD = ApiMethod.UPDATE_ENTRY

    class ApiParams(Enum):
        TITLE = "title"
        STARRED = "starred"
        ARCHIVE = "archive"

    def __init__(self, config, entry_id, params):
        Api.__init__(self, config)
        self.entry_id = entry_id
        self.params = params

    def _make_request(self, request):
        return self._request_patch(request)

    def _get_api_url(self):
        entry_id = self._validate_entry_id(self.entry_id)
        return self._build_url(UpdateEntry.API_METHOD).format(entry_id)

    def _get_data(self):
        ApiParams = self.ApiParams
        data = {}
        if Params.TITLE in self.params and self.params[Params.TITLE]:
            data[ApiParams.TITLE.value] = self.params[Params.TITLE]
        if Params.STAR in self.params:
            self._put_bool_param(data, Params.STAR, ApiParams.STARRED)
        if Params.READ in self.params:
            self._put_bool_param(data, Params.READ, ApiParams.ARCHIVE)
        if not data:
            raise ValueException("The data object is empty")
        return data



# -*- coding: utf-8 -*-


from wallabag.api.api import Api, ApiMethod
from wallabag.config import Options, Sections


class ApiToken(Api):

    def __init__(self, config):
        Api.__init__(self, config)
        self.skip_auth = True

    def _get_api_url(self):
        return self._build_url(ApiMethod.TOKEN)

    def _make_request(self, request):
        return self._request_get(request)

    def _get_params(self):
        return {
            'grant_type': "password",
            'client_id': self.config.get(Sections.OAUTH2, Options.CLIENT),
            'client_secret': self.config.get(Sections.OAUTH2, Options.SECRET),
            'username': self.config.get(Sections.API, Options.USERNAME),
            'password': self.config.get(Sections.API, Options.PASSWORD)
        }
